/******************************************************************************
*       Author          Neha Sharma
*       Organization    Fermi National Accelerator Laboratory
*       Description     This class implements the SAZ's decision making logic
*       Email           neha@fnal.gov
******************************************************************************/
package gov.fnal.vox.saz.bl;

import java.io.BufferedReader;
import java.lang.String;
import java.lang.Throwable;

import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;

import gov.fnal.vox.saz.entity.User;
import gov.fnal.vox.saz.entity.CA;
import gov.fnal.vox.saz.entity.VO;
import gov.fnal.vox.saz.entity.Role;
//import gov.fnal.vox.saz.bl.util.SAZUtil;
import gov.fnal.vox.saz.bl.util.SAZConfiguration;
import gov.fnal.vox.saz.bl.HibernateUtil;
import gov.fnal.vox.saz.dao.impl.hibernate.UserDaoImpl;
import gov.fnal.vox.saz.dao.impl.hibernate.CADaoImpl;
import gov.fnal.vox.saz.dao.impl.hibernate.VODaoImpl;
import gov.fnal.vox.saz.dao.impl.hibernate.RoleDaoImpl;
import gov.fnal.vox.saz.dao.impl.hibernate.BaseDaoImpl;
import gov.fnal.vox.saz.exception.AddUserException;
import gov.fnal.vox.saz.exception.FindUserException;
import gov.fnal.vox.saz.exception.FindUserUnexpectedException;
import gov.fnal.vox.saz.exception.ListUsersException;
import gov.fnal.vox.saz.exception.AddCAException;
import gov.fnal.vox.saz.exception.FindCAException;
import gov.fnal.vox.saz.exception.FindCAUnexpectedException;
import gov.fnal.vox.saz.exception.AddVOException;
import gov.fnal.vox.saz.exception.FindVOException;
import gov.fnal.vox.saz.exception.FindVOUnexpectedException;
import gov.fnal.vox.saz.exception.AddRoleException;
import gov.fnal.vox.saz.exception.FindRoleException;
import gov.fnal.vox.saz.exception.FindRoleUnexpectedException;
import org.hibernate.exception.ConstraintViolationException;
import org.hibernate.exception.JDBCConnectionException;

public class SAZAPIImpl{	
	private SAZConfiguration confObject;
	//private Session session;
	private BaseDaoImpl baseDaoImpl;
	private String incomingUserDN;
	private String incomingUserCA;
	private String incomingUserVO;
	private String incomingUserRole;
	private UserDaoImpl userDaoImpl;
	private CADaoImpl caDaoImpl;
	private VODaoImpl voDaoImpl;
	private RoleDaoImpl roleDaoImpl;	
	//private SAZUtil utilObject;
	private static final transient Logger logger=Logger.getLogger(SAZAPIImpl.class.getName());
			
	public SAZAPIImpl(){
		confObject=SAZConfiguration.getInstance();
		/*session=HibernateUtil.getSession();
		logger.debug("CHECK Constructor: Is Session Open: "+session.isOpen());
		userDaoImpl=new UserDaoImpl();
		voDaoImpl=new VODaoImpl();
		caDaoImpl=new CADaoImpl();
		roleDaoImpl=new RoleDaoImpl();
		userDaoImpl.setSession(session);
		voDaoImpl.setSession(session);
		caDaoImpl.setSession(session);
		roleDaoImpl.setSession(session);*/
	}
	public String getIsUserCertSerialNumBanEnabled(){
		return confObject.getDefaultSAZBanUserCertSerialNum();
	}
	public String getIsUserProxyLifetimeBanEnabled(){
                return confObject.getDefaultSAZBanUserProxyLifetime();
        }
	/*public List<User> listUsers(){
		Transaction tx=null;
		List<User> allUsers=null;
		try{
			tx=session.beginTransaction();
			allUsers=userDaoImpl.list();
			if (tx != null) tx.commit();
			return allUsers;
		}catch(ListUsersException lue){
			if (tx != null) tx.rollback();
			logger.error(lue.getMessage());
                        lue.printStackTrace();
		}finally{
                        if(session.isOpen()) session.close();
		}
		return allUsers;
	}*/
	public void addUser(String incomingUserDN,String defUserEnabled,String defUserTrusted){	
		Session newSession=HibernateUtil.getNewSession();
		/*int retry_num=confObject.getDefaultSAZQueryRetryNum();
		try{
			for(int i=0;i<retry_num;i++){
				Transaction utx=session.beginTransaction();	
				try{
					logger.debug("Going to add User..");
					userDaoImpl.add(session,incomingUserDN,defUserEnabled,defUserTrusted);
					utx.commit();
					return;
				}catch(AddUserException aue){
                		        if (utx != null) utx.rollback();
                        		logger.error(aue.getMessage());
		                        aue.printStackTrace();
				}
			}	
		}finally{
			if(session.isOpen()) session.close();
		}*/
		Transaction utx=newSession.beginTransaction();
		try{
                	logger.debug("Going to add User..");
                	userDaoImpl.add(newSession,incomingUserDN,defUserEnabled,defUserTrusted);
                	utx.commit();
                	return;
                }catch(AddUserException aue){
                	if (utx != null) utx.rollback();
                       	logger.error(aue.getMessage());
                	aue.printStackTrace();
		}finally{
                        if(newSession.isOpen()) newSession.close();
                }
	}
	public void addVO(String incomingUserVO,String defVOEnabled,String defVOTrusted){
                Session newSession=HibernateUtil.getNewSession();
                Transaction vtx=newSession.beginTransaction();
                try{
			logger.debug("Going to add VO..");
                        voDaoImpl.add(newSession,incomingUserVO,defVOEnabled,defVOTrusted);
                        vtx.commit();
                }catch(AddVOException ave){
                        if (vtx != null) vtx.rollback();
                        logger.error(ave.getMessage());
                        ave.printStackTrace();
                }finally{
                        if(newSession.isOpen()) newSession.close();
                }
        }
	public void addCA(String incomingUserCA,String defCAEnabled,String defCATrusted){
                Session newSession=HibernateUtil.getNewSession();
                Transaction ctx=newSession.beginTransaction();
                try{
			logger.debug("Going to add CA..");	
                        caDaoImpl.add(newSession,incomingUserCA,defCAEnabled,defCATrusted);
                        ctx.commit();
                }catch(AddCAException ace){
                        if (ctx != null) ctx.rollback();
                        logger.error(ace.getMessage());
                        ace.printStackTrace();
                }finally{
                        if(newSession.isOpen()) newSession.close();
                }
        }
	public void addRole(String incomingUserRole,String defRoleEnabled,String defRoleTrusted){
                Session newSession=HibernateUtil.getNewSession();
                Transaction rtx=newSession.beginTransaction();
                try{
			logger.debug("Going to add Role..");
                        roleDaoImpl.add(newSession,incomingUserRole,defRoleEnabled,defRoleTrusted);
                        rtx.commit();
                }catch(AddRoleException are){
                        if (rtx != null) rtx.rollback();
                        logger.error(are.getMessage());
                        are.printStackTrace();
                }finally{
                        if(newSession.isOpen()) newSession.close();
                }
        }
	public boolean compareProxyLifetime(long lifetimeIncomingProxy){
		String defaultLifetime=confObject.getDefaultUserProxyLifetime();
		//int  lifetimeAllowed=Integer.parseInt(defaultLifetime);	
		long lifetimeAllowed=Long.valueOf(defaultLifetime);
		if((lifetimeAllowed > 0) && (lifetimeIncomingProxy > lifetimeAllowed)){
			return true;
		}else{
			return false;
		}	
	}
	public String isEnabled(String incomingUserDN, String incomingUserCA, String incomingUserVO, String incomingUserRole){
		//setting the default response from SAZ as "no"
		String sazDecision="N";
		Transaction tx=null;
		//Need a unique session each time a request comes in
		//Session session=HibernateUtil.getSession();
                Session session=HibernateUtil.getNewSession();
		userDaoImpl=new UserDaoImpl();
                voDaoImpl=new VODaoImpl();
                caDaoImpl=new CADaoImpl();
                roleDaoImpl=new RoleDaoImpl();
                //userDaoImpl.setSession(session);
                //voDaoImpl.setSession(session);
                //caDaoImpl.setSession(session);
                //roleDaoImpl.setSession(session);
		try{
			User user=null;
			CA ca=null;
			VO vo=null;
			Role role=null;
			boolean isGridProxy=false;
			//Read the default enable/trusted settings from the configuration file
			String defUserEnabled=confObject.getDefaultSAZUserEnabled();
			String defUserTrusted=confObject.getDefaultSAZUserTrusted();
			String defCAEnabled=confObject.getDefaultSAZCAEnabled();
			String defCATrusted=confObject.getDefaultSAZCATrusted();
			String defVOEnabled=confObject.getDefaultSAZVOEnabled();
			String defVOTrusted=confObject.getDefaultSAZVOTrusted();
			String defRoleEnabled=confObject.getDefaultSAZRoleEnabled();
			String defRoleTrusted=confObject.getDefaultSAZRoleTrusted();
			String userEnabled=defUserEnabled;
			String userTrusted=defUserTrusted;
			String caEnabled=defCAEnabled;
			String caTrusted=defCATrusted;
			String voEnabled=defVOEnabled;
			String voTrusted=defVOTrusted;
			String roleEnabled=defRoleEnabled;
			String roleTrusted=defRoleTrusted;
			//Escape any single quotes present in the incoming user DN
			incomingUserDN=incomingUserDN.replaceAll("\'","\\\\\'");
                        
			logger.info("User "+incomingUserDN);
                        logger.info("CA "+incomingUserCA);
                        logger.info("VO "+incomingUserVO);
                        logger.info("Role "+incomingUserRole);

			//Validate the input provided by Client before executing queries against the DB
			/*try{
				utilObject.validateDN(incomingUserDN);
				utilObject.validateDN(incomingUserCA);
				utilObject.validateVO(incomingUserVO);
				utilObject.validateRole(incomingUserRole);	
			}catch(Exception e){
				logger.error(e.getMessage());
			}*/
			// Get the user info from DB or add if it doesn't exist already
			if((incomingUserVO == null) && (incomingUserRole == null) || (incomingUserVO.equals("NULL") && incomingUserRole.equals("NULL"))){
				isGridProxy=true;
			}
			logger.debug("Is Session Open: "+session.isOpen());
			tx=session.beginTransaction();
			logger.debug("Is Transaction Active: "+tx.isActive());
			user=userDaoImpl.find(session,incomingUserDN);
			if(user != null){
				userEnabled=user.getEnabled();
				userTrusted=user.getTrusted();
			}else{
				try{
					/*Modified the call to add for all 4 entities to account for case
					when parallel requests come in and try to add an entry in MySQL DB	
					with same primary  key. In such case, we log the exception, but 
					look for entity again and go ahead with our business*/
					addUser(incomingUserDN,defUserEnabled,defUserTrusted);
					//userDaoImpl.add(session,incomingUserDN,defUserEnabled,defUserTrusted);
				}catch(ConstraintViolationException cve){
					logger.error(cve.getMessage());		
				}
			}
			// Get the CA info from DB or add it if doesn't exist already
                        ca=caDaoImpl.find(session,incomingUserCA);
                        if( ca != null){
                                caEnabled=ca.getEnabled();
                                caTrusted=ca.getTrusted();
                        }else{
				try{
                               		addCA(incomingUserCA,defCAEnabled,defCATrusted);
					//caDaoImpl.add(session,incomingUserCA,defCAEnabled,defCATrusted);
				}catch(ConstraintViolationException cve){
					logger.error(cve.getMessage());
                                }
                        }
			//If its a voms proxy, get the VO and Role info from DB or add them if they don't exist already
			if(!isGridProxy){
                	        vo=voDaoImpl.find(session,incomingUserVO);
                        	if( vo != null){
                                	voEnabled=vo.getEnabled();
                        	        voTrusted=vo.getTrusted();	
				}else{
					try{
                                        	addVO(incomingUserVO,defVOEnabled,defVOTrusted);
                                		//voDaoImpl.add(session,incomingUserVO,defVOEnabled,defVOTrusted);
					}catch(ConstraintViolationException cve){
                                        	logger.error(cve.getMessage());
                                	}
				}
				role=roleDaoImpl.find(session,incomingUserRole);
        	                if( role != null){
                	                roleEnabled=role.getEnabled();
                        	        roleTrusted=role.getTrusted();
				}else{
					try{	
						addRole(incomingUserRole,defRoleEnabled,defRoleTrusted);
                        	        	//roleDaoImpl.add(session,incomingUserRole,defRoleEnabled,defRoleTrusted);
					}catch(ConstraintViolationException cve){
                                                logger.error(cve.getMessage());
                                        }
                        	}
			}
			//commit the transaction
			if (tx != null) tx.commit();
			/* If its a simple grid proxy, send yes, only if the user is both enabled and trusted
			   and the CA is enabled. If its a voms extended proxy, send yes, only if the user,ca,
			   vo and role are all enabled*/	
			if(isGridProxy){
				if((userEnabled.equals("Y"))&&(userTrusted.equals("Y"))&&(caEnabled.equals("Y"))){
					sazDecision="Y";
				}else{
					sazDecision="N";
				}
			}else{
				if(userEnabled.equals("Y")&&caEnabled.equals("Y")&&voEnabled.equals("Y")&&roleEnabled.equals("Y")){
					sazDecision="Y";
				}else{
					sazDecision="N";
				}
			}
			return sazDecision;
		}catch(FindUserException fue){
			if (tx != null) tx.rollback();
			logger.error(fue.getMessage());
			fue.printStackTrace();
		}catch(FindUserUnexpectedException fuue){
			if (tx != null) tx.rollback();
			logger.error(fuue.getMessage());
			fuue.printStackTrace();
                }catch(FindCAException fce){
			if (tx != null) tx.rollback();
                        logger.error(fce.getMessage());
                        fce.printStackTrace();
                }catch(FindCAUnexpectedException fcue){
			if (tx != null) tx.rollback();
                        logger.error(fcue.getMessage());
                        fcue.printStackTrace();
                }catch(FindVOException fve){
			if (tx != null) tx.rollback();
                        logger.error(fve.getMessage());
                        fve.printStackTrace();
                }catch(FindVOUnexpectedException fvue){
			if (tx != null) tx.rollback();
                        logger.error(fvue.getMessage());
                        fvue.printStackTrace();
                }catch(FindRoleException fre){
			if (tx != null) tx.rollback();
                        logger.error(fre.getMessage());
                        fre.printStackTrace();
                }catch(FindRoleUnexpectedException frue){
			if (tx != null) tx.rollback();
                        logger.error(frue.getMessage());
                        frue.printStackTrace();
		}catch(JDBCConnectionException jdbcce){
			//If MYSQL server goes down, return "N" to incoming requests
			if (tx != null) tx.rollback();
			logger.error(jdbcce.getMessage());
			jdbcce.printStackTrace();
			logger.error("MySQL Down! Sending to client "+sazDecision);
			return sazDecision;
		}catch(Exception e){	
			if (tx != null) tx.rollback();
			logger.error(e.getMessage());
			e.printStackTrace();
		}finally{
			if(session.isOpen()) session.close();
		}
		return sazDecision;
	}
}
