/*****************************************************************************
*
*	File		SAZConfiguration.java
*	Author		Valery Sergeev, Neha Sharma
*	Date		Tue Sept 22 10:29:15 CDT 2009
*	Organization	Fermi National Accelerator Laboratory
*	Description	To get the default SAZ Server Configuration
*	Email		neha@fnal.gov
*
*
******************************************************************************/

package gov.fnal.vox.saz.bl.util;

import java.io.IOException;
import java.util.Vector;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import org.xml.sax.SAXException;
import org.apache.log4j.Logger;

import org.globus.gsi.TrustedCertificates;
 
public class SAZConfiguration{	
	private int numberOfElements;
	private int crlCounter;
	private long refreshInterval; 
	private Document dom;
	private static SAZConfiguration ref;
	private final ReadWriteLock rwl=new ReentrantReadWriteLock();
	private Lock writeLock=rwl.writeLock();
	private Lock readLock=rwl.readLock();
	private static final transient Logger logger=Logger.getLogger(SAZConfiguration.class.getName());
	private String SAZ_USER_TRUSTED;
	private String SAZ_USER_ENABLED;
	private String SAZ_CA_TRUSTED;
	private String SAZ_CA_ENABLED;
	private String SAZ_VO_TRUSTED;
	private String SAZ_VO_ENABLED;
	private String SAZ_ROLE_TRUSTED;
	private String SAZ_ROLE_ENABLED;
	private String SAZ_SERVER_CERT;
        private String SAZ_SERVER_KEY;
        private String SAZ_SERVER_PORT;
	private String SAZ_BAN_USERCERT_SERIALNUM;
	private String SAZ_BAN_USERPROXY_LIFETIME;
	private String USERPROXY_LIFETIME;
	private String SAZ_SERVER_POOL_SIZE;
	private String SAZ_CRL_REFRESH_INTERVAL;
        private String CA_DIR;
	private TrustedCertificates TRUSTED_CERTS;
	
	private SAZConfiguration(String fileName){
		parseSAZConfFile(fileName);
	}
	public static synchronized SAZConfiguration getInstance(String fileName){
		if(ref==null){
			ref=new SAZConfiguration(fileName);
		}
		return ref;
	}
	public static synchronized SAZConfiguration getInstance(){
		return ref;
	}		
	private void parseSAZConfFile(String fileName){
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try{
			//Using factory get an instance of document builder
			DocumentBuilder db = dbf.newDocumentBuilder();
			//parse using builder to get DOM representation of the XML file
			dom = db.parse(fileName);
			System.out.println("XML File = "+fileName);
		}catch(ParserConfigurationException pce){
			pce.printStackTrace();
		}catch(SAXException se){
			se.printStackTrace();
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
		setDefaultSAZConfValues();
	}
	private void setDefaultSAZConfValues(){
		Element docEle = dom.getDocumentElement();
		NodeList nl = docEle.getElementsByTagName("SAZ_USER_TRUSTED");
		Element el = (Element)nl.item(0);					
		SAZ_USER_TRUSTED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_USER_ENABLED");
		el = (Element)nl.item(0);			
		SAZ_USER_ENABLED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_CA_TRUSTED");
		el = (Element)nl.item(0);			
		SAZ_CA_TRUSTED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_CA_ENABLED");
		el = (Element)nl.item(0);			
		SAZ_CA_ENABLED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_VO_TRUSTED");
		el = (Element)nl.item(0);			
		SAZ_VO_TRUSTED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_VO_ENABLED");
		el = (Element)nl.item(0);			
		SAZ_VO_ENABLED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_ROLE_TRUSTED");
		el = (Element)nl.item(0);			
		SAZ_ROLE_TRUSTED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_ROLE_ENABLED");
		el = (Element)nl.item(0);			
		SAZ_ROLE_ENABLED=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_SERVER_CERT");
                el = (Element)nl.item(0);
                SAZ_SERVER_CERT=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_SERVER_KEY");
                el = (Element)nl.item(0);
                SAZ_SERVER_KEY=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_BAN_USERCERT_SERIALNUM");
		el = (Element)nl.item(0);
		SAZ_BAN_USERCERT_SERIALNUM=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_BAN_USERPROXY_LIFETIME");
                el = (Element)nl.item(0);
                SAZ_BAN_USERPROXY_LIFETIME=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("USERPROXY_LIFETIME");
                el = (Element)nl.item(0);
                USERPROXY_LIFETIME=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_SERVER_PORT");
                el = (Element)nl.item(0);
                SAZ_SERVER_PORT=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("CA_DIR");
                el = (Element)nl.item(0);
                CA_DIR=el.getLastChild().getNodeValue();	
		nl = docEle.getElementsByTagName("SAZ_CRL_REFRESH_INTERVAL");
                el = (Element)nl.item(0);
                SAZ_CRL_REFRESH_INTERVAL=el.getLastChild().getNodeValue();
		nl = docEle.getElementsByTagName("SAZ_SERVER_POOL_SIZE");
                el = (Element)nl.item(0);
                SAZ_SERVER_POOL_SIZE=el.getLastChild().getNodeValue();		
		System.out.println("SAZ_SERVER_CERT = "+SAZ_SERVER_CERT);
		System.out.println("SAZ_SERVER_KEY = "+SAZ_SERVER_KEY);
		System.out.println("SAZ_SERVER_PORT = "+SAZ_SERVER_PORT);
		System.out.println("CA_DIR = "+CA_DIR);
		System.out.println("SAZ_CRL_REFRESH_INTERVAL = "+SAZ_CRL_REFRESH_INTERVAL);
		System.out.println("SAZ_USER_TRUSTED = "+SAZ_USER_TRUSTED);
		System.out.println("SAZ_USER_ENABLED = "+SAZ_USER_ENABLED);
		System.out.println("SAZ_CA_TRUSTED = "+SAZ_CA_TRUSTED);
		System.out.println("SAZ_CA_ENABLED = "+SAZ_CA_ENABLED);
		System.out.println("SAZ_VO_TRUSTED = "+SAZ_VO_TRUSTED);
		System.out.println("SAZ_VO_ENABLED = "+SAZ_VO_ENABLED);
		System.out.println("SAZ_ROLE_TRUSTED = "+SAZ_ROLE_TRUSTED);
		System.out.println("SAZ_ROLE_ENABLED = "+SAZ_ROLE_ENABLED);
		System.out.println("SAZ_SERVER_POOL_SIZE = "+SAZ_SERVER_POOL_SIZE);
		System.out.println("SAZ_BAN_USERCERT_SERIALNUM = "+SAZ_BAN_USERCERT_SERIALNUM);
		System.out.println("SAZ_BAN_USERPROXY_LIFETIME = "+SAZ_BAN_USERPROXY_LIFETIME);
		System.out.println("USERPROXY_LIFETIME = "+USERPROXY_LIFETIME);
	}
	public String getDefaultSAZUserTrusted(){
		return SAZ_USER_TRUSTED;
	}
	public String getDefaultSAZUserEnabled(){
                return SAZ_USER_ENABLED;
        }
        public String getDefaultSAZCATrusted(){
                return SAZ_CA_TRUSTED;
        }
        public String getDefaultSAZCAEnabled(){
                return SAZ_CA_ENABLED;
        }
        public String getDefaultSAZVOTrusted(){
                return SAZ_VO_TRUSTED;
        }
        public String getDefaultSAZVOEnabled(){
                return SAZ_VO_ENABLED;
        }
        public String getDefaultSAZRoleTrusted(){
                return SAZ_ROLE_TRUSTED;
        }
        public String getDefaultSAZRoleEnabled(){
                return SAZ_ROLE_ENABLED;
        }
	public String getDefaultSAZServerCert(){
                return SAZ_SERVER_CERT;
        }
        public String getDefaultSAZServerKey(){
                return SAZ_SERVER_KEY;
        }
	public String getDefaultSAZBanUserCertSerialNum(){
		return SAZ_BAN_USERCERT_SERIALNUM;
	}
	public String getDefaultSAZBanUserProxyLifetime(){
                return SAZ_BAN_USERPROXY_LIFETIME;
        }
	public String getDefaultUserProxyLifetime(){
                return USERPROXY_LIFETIME;
        }
        public int getDefaultSAZServerPort(){
                return Integer.parseInt(SAZ_SERVER_PORT);
        }
        public String getDefaultCADir(){
                return CA_DIR;
        }
	public int getDefaultSAZServerPoolSize(){
		return Integer.parseInt(SAZ_SERVER_POOL_SIZE);
	}
	public long getDefaultSAZCRLRefreshInterval(){
		return (1000*Long.parseLong(SAZ_CRL_REFRESH_INTERVAL));
	}
	private int getCrlCounter(){
		crlCounter+=crlCounter;
		return crlCounter;
	}
	public void loadTrustedCerts(){
		logger.info("Loading Trusted Certificates..");
		logger.debug("Acquiring the Write Lock ..");
		writeLock.lock();
		try{
			logger.debug("Going to invoking load function. Counter is: "+getCrlCounter());	
			TRUSTED_CERTS=TrustedCertificates.load(getDefaultCADir());
			logger.debug("Finished invoking load function..");
			logger.debug("Certificates CHANGED: "+TRUSTED_CERTS.isChanged());
			//logger.debug("Certificate: "+TRUSTED_CERTS.toString());
		}catch(Exception e){
			logger.error("Exception occured while loading Trusted Certificates..");
			logger.error(e.getMessage());
		}
		finally{
			logger.debug("Releasing the Write Lock..");
			writeLock.unlock();
		}
		logger.info("Finished loading Trusted Certificates..");
	}
	public TrustedCertificates getTrustedCerts(){
		readLock.lock();
		logger.debug("Read Lock acquired..");
		try{
			logger.debug("Returning Trusted Certificates..");	
			return 	TRUSTED_CERTS;
		}finally{
			readLock.unlock();
			logger.debug("Read Lock released..");
		}
	}
}
