/*****************************************************************************
*
*       File            TidiedDailyRollingFileAppender.java
*       Date            Tue Jan 10 14:25:17 CDT 2012
*       Organization    Fermi National Accelerator Laboratory
*       Description     A class for removing logs older than 30 days
*	Note		This class has been copied from gratia code 
*
******************************************************************************/
package gov.fnal.vox.saz.bl.util;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.apache.log4j.Layout;
import org.apache.log4j.helpers.LogLog;
import org.apache.log4j.spi.LoggingEvent;
import org.apache.log4j.DailyRollingFileAppender;

public class TidiedDailyRollingFileAppender extends DailyRollingFileAppender{
	private int maxAgeDays = -1;
	private String file="";
	private String conversionPattern="%d{yyyy-MM-dd HH:mm:ss} :%-5p:%t: %m%n";
	private String datePattern=".yyyy-MM-dd";
	static boolean initialized=false;
	public TidiedDailyRollingFileAppender(){
        	// LogLog.warn("Instantiated a TidiedDailyRollingFileAppender");
	}
	public TidiedDailyRollingFileAppender(Layout layout, String filename,String datePattern) throws IOException{
    		super(layout, filename, datePattern);
		// LogLog.warn("Instantiated a TidiedDailyRollingFileAppender");
	}
	public void setMaxAgeDays(int value){
		this.maxAgeDays = value;
	}
	public int getMaxAgeDays(){
        	return this.maxAgeDays;
	}
	/*public void setFile(String name){
	        this.file = name;
        }
        public String getFile(){
                return this.file;
        }
	public void setDatePattern(String datePattern){
		this.datePattern=datePattern;
	}
	public String getDatePattern(){
		return this.datePattern;
	}
	public void setConversionPattern(String conversionPattern){
		this.conversionPattern=conversionPattern;
	}
	public String getConversionPattern(){
		return this.conversionPattern;
	}*/
	public void activateOptions() {
        	super.activateOptions();
    	}
	protected synchronized void subAppend(LoggingEvent event){
		super.subAppend(event);
		checkAndDeleteFiles();
	}
	public File[] checkOldFiles(){
        	// LogLog.warn("Checking for old files based on " + fileName);
        	File file = new File(fileName);
        	DatedFileFilter ff = new DatedFileFilter(getDatePattern(),file.getName(),maxAgeDays);
		return file.getParentFile().listFiles(ff);
    	}
    	private void checkAndDeleteFiles() {
        	for (File dFile : checkOldFiles()) {
            		try {
               			LogLog.debug("Deleting old log file " + dFile.getName());
            		} catch (Exception e) {
               			System.err.print("Caught an exception in TidiedDailyRollingFileAppender::checkAndDeleteFiles while trying to log an information message:"+e);
               			System.out.print("Caught an exception in TidiedDailyRollingFileAppender::checkAndDeleteFiles while trying to log an information message:"+e);
               			e.printStackTrace();
            		}
            		dFile.delete();
        	}
    	}
	class DatedFileFilter implements FilenameFilter {
		private String filename;
        	private SimpleDateFormat sdf;
        	private int maxAgeDays;

        	public DatedFileFilter(String datePattern, String filename, int maxAgeDays) {
            		// LogLog.warn("Instantiated a DatedFileFilter(" + datePattern + ", " + filename + ", " + maxAgeDays + ")");
            		sdf = new SimpleDateFormat(datePattern);
            		this.filename = filename;
            		this.maxAgeDays = maxAgeDays;
        	}
		public boolean accept(File dir, String name) {
            		// LogLog.warn("DatedFileFilter.accept(" + dir + ", " + name + ")");
            		boolean result = false;
            		if (maxAgeDays < 0) {
                		// LogLog.warn("DatedFileFilter.accept(" + dir + ", " + name + "): maxAgeDays < 0");
                		return result; // Quick kick out
            		}	
            		if (!name.startsWith(filename)) {
                		// LogLog.warn("DatedFileFilter.accept(" + dir + ", " + name + "): no match to " + filename);
                		return result; // Not our file
            		}
            		ParsePosition pos = new ParsePosition(filename.length());
            		Date filedate = sdf.parse(name, pos);
            		// LogLog.warn("DatedFileFilter.accept(" + dir + ", " + name + "): SimpleDateFormat.parse() returned " +// ((filedate == null)?"null":filedate.toString()) + // ", error pos = " + pos.getErrorIndex());
			if ((filedate == null) || (pos.getErrorIndex() > -1)) {
				return result;
            		}
            		GregorianCalendar fileCal = new GregorianCalendar();
            		fileCal.setTime(filedate);
            		GregorianCalendar cmpCal = new GregorianCalendar();
            		cmpCal.add(Calendar.DAY_OF_MONTH, -1 * maxAgeDays);
            		// LogLog.warn("DatedFileFilter.accept(" + dir + ", " + name + "): comparing " + filedate + " to " + cmpCal.getTime());
            		if (cmpCal.after(fileCal)) {
                		// LogLog.warn("DatedFileFilter.accept(" + dir + ", " + name + "): identified old file " + name);
                		result = true;
            		}
            		return result;
        	}
	}
}

