/*****************************************************************************
*       File            AdminDaoImpl.java
*       Author          Neha Sharma
*       Date            August 26' 2009
*       Organization    Fermi National Accelerator Laboratory
*       Description     The implementation of Admin DAO interface.
*       Email           neha@fnal.gov
******************************************************************************/
package gov.fnal.vox.saz.dao.impl.hibernate;

import java.util.List;
import gov.fnal.vox.saz.entity.Admin;
import org.hibernate.Session;
import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.HibernateException;
import gov.fnal.vox.saz.dao.AdminDao;
import gov.fnal.vox.saz.exception.AddAdminException;
import gov.fnal.vox.saz.exception.BanAdminException;
import gov.fnal.vox.saz.exception.UnbanAdminException;
import gov.fnal.vox.saz.exception.FindAdminException;
import gov.fnal.vox.saz.exception.ListAdminsException;
import gov.fnal.vox.saz.exception.FindAdminUnexpectedException;
import org.hibernate.exception.ConstraintViolationException;

public class AdminDaoImpl extends BaseDaoImpl implements AdminDao{
	private static final transient Logger logger=Logger.getLogger(AdminDaoImpl.class.getName());
	public void add(Session session,String dn,String enabled,String principal) throws AddAdminException{
		logger.debug("AddAdmin: Is Session Open: "+session.isOpen());
		try{
			Admin newAdmin=new Admin();
			newAdmin.setDn(dn);
			newAdmin.setEnabled(enabled);
			newAdmin.setPrincipal(principal);
			session.save(newAdmin);
		}catch(HibernateException e){
			throw new AddAdminException("Exception: Add New Admin: "+e.getMessage());
 		}
	}
	public Admin find(Session session,String dn) throws FindAdminException, FindAdminUnexpectedException{
		Admin toReturn=null;
		logger.debug("FindAdmin: Is Session Open: "+session.isOpen());	
		try{
 			Query query=(session.createQuery("from Admin where dn = :var"));	
			query.setString("var",dn);
			query.setCacheable(true);
			Object o=query.uniqueResult();
			if(o != null){
				if(o instanceof Admin){
					toReturn=(Admin)o;
					return toReturn;
				}else{
					throw new FindAdminUnexpectedException("Unexpected Exception: Find Admin: ");
				}
			}else{
				return null;
			}
 		}catch(HibernateException e){
 			throw new FindAdminException("Exception: Find Admin: "+e.getMessage());
		}
	}
	public List<Admin> list(Session session) throws ListAdminsException{
		List<Admin> result=null;
		try{
			Query query=(session.createQuery("from Admin"));
			result=query.list();
			return result;	
		}catch(HibernateException e){
			throw new ListAdminsException("Exception: ListAdmins: "+e.getMessage());
		}
	}
	public List<Admin> listAllBannedAdmins(Session session) throws ListAdminsException{
		List<Admin> result=null;
		try{
                        Query query=(session.createQuery("from Admin where enabled = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListAdminsException("Exception: ListAllBannedAdmins: "+e.getMessage());
                }
        }
	public List<Admin> listAllUnbannedAdmins(Session session) throws ListAdminsException{
                List<Admin> result=null;
                try{
                        Query query=(session.createQuery("from Admin where enabled = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result); 
                }catch(HibernateException e){
                        throw new ListAdminsException("Exception: ListAllUnbannedAdmins: "+e.getMessage());
                }
        }
	public void banAdminBasedOnDN(Session session,String dn) throws BanAdminException{
                try{
                        Query query=(session.createQuery("update Admin set enabled = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new BanAdminException("Exception: BanAdminBasedOnDN: "+e.getMessage());
                }
        }
	public void unbanAdminBasedOnDN(Session session,String dn) throws UnbanAdminException{
                try{
                        Query query=(session.createQuery("update Admin set enabled = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new UnbanAdminException("Exception: UnbanAdminBasedOnDN: "+e.getMessage());
                }
        }
}
