/*****************************************************************************
*       File            CADaoImpl.java
*       Author          Neha Sharma
*       Date            August 26' 2009
*       Organization    Fermi National Accelerator Laboratory
*       Description     The implementation of CA DAO interface.
*       Email           neha@fnal.gov
******************************************************************************/
package gov.fnal.vox.saz.dao.impl.hibernate;

import java.util.List;
import gov.fnal.vox.saz.entity.CA;
import org.hibernate.Session;
import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.HibernateException;
import gov.fnal.vox.saz.dao.CADao;
import gov.fnal.vox.saz.exception.AddCAException;
import gov.fnal.vox.saz.exception.BanCAException;
import gov.fnal.vox.saz.exception.UnbanCAException;
import gov.fnal.vox.saz.exception.TrustCAException;
import gov.fnal.vox.saz.exception.DistrustCAException;
import gov.fnal.vox.saz.exception.FindCAException;
import gov.fnal.vox.saz.exception.ListCAsException;
import gov.fnal.vox.saz.exception.FindCAUnexpectedException;
import org.hibernate.exception.ConstraintViolationException;

public class CADaoImpl extends BaseDaoImpl implements CADao{
	private static final transient Logger logger=Logger.getLogger(CADaoImpl.class.getName());
	public void add(Session session,String dn,String enabled,String trusted) throws AddCAException{
		logger.debug("AddCA: Is Session Open: "+session.isOpen());
		try{
			CA newCA=new CA();
			newCA.setDn(dn);
			newCA.setEnabled(enabled);
			newCA.setTrusted(trusted);
			session.save(newCA);
		}catch(HibernateException e){
			throw new AddCAException("Exception: Add New CA: "+e.getMessage());
 		}
	}
	public CA find(Session session,String dn) throws FindCAException, FindCAUnexpectedException{
		CA toReturn=null;
		logger.debug("FindCA: Is Session Open: "+session.isOpen());	
		try{
 			Query query=(session.createQuery("from CA where dn = :var"));	
			query.setString("var",dn);
			query.setCacheable(true);
			Object o=query.uniqueResult();
			if(o != null){
				if(o instanceof CA){
					toReturn=(CA)o;
					return toReturn;
				}else{
					throw new FindCAUnexpectedException("Unexpected Exception: Find CA: ");
				}
			}else{
				return null;
			}
 		}catch(HibernateException e){
 			throw new FindCAException("Exception: Find CA: "+e.getMessage());
		}
	}
	public List<CA> list(Session session) throws ListCAsException{
		List<CA> result=null;
		try{
			Query query=(session.createQuery("from CA"));
			result=query.list();
			return result;	
		}catch(HibernateException e){
			throw new ListCAsException("Exception: ListCAs: "+e.getMessage());
		}
	}
	public List<CA> listAllBannedCAs(Session session) throws ListCAsException{
		List<CA> result=null;
		try{
                        Query query=(session.createQuery("from CA where enabled = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListCAsException("Exception: ListAllBannedCAs: "+e.getMessage());
                }
        }
	public List<CA> listAllUnbannedCAs(Session session) throws ListCAsException{
                List<CA> result=null;
                try{
                        Query query=(session.createQuery("from CA where enabled = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result); 
                }catch(HibernateException e){
                        throw new ListCAsException("Exception: ListAllUnbannedCAs: "+e.getMessage());
                }
        }
	public List<CA> listAllTrustedCAs(Session session) throws ListCAsException{
                List<CA> result=null;
                try{
                        Query query=(session.createQuery("from CA where trusted = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListCAsException("Exception: ListAllTrustedCAs: "+e.getMessage());
                }
        }
	public List<CA> listAllDistrustedCAs(Session session) throws ListCAsException{
                List<CA> result=null;
                try{
                        Query query=(session.createQuery("from CA where trusted = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListCAsException("Exception: ListAllDistrustedCAs: "+e.getMessage());
                }
        }
	public void banCABasedOnDN(Session session,String dn) throws BanCAException{
                try{
                        Query query=(session.createQuery("update CA set enabled = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new BanCAException("Exception: BanCABasedOnDN: "+e.getMessage());
                }
        }
	public void unbanCABasedOnDN(Session session,String dn) throws UnbanCAException{
                try{
                        Query query=(session.createQuery("update CA set enabled = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new UnbanCAException("Exception: UnbanCABasedOnDN: "+e.getMessage());
                }
        }
	public void trustCABasedOnDN(Session session,String dn) throws TrustCAException{
                try{
                        Query query=(session.createQuery("update CA set trusted = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new TrustCAException("Exception: TrustCABasedOnDN: "+e.getMessage());
                }
        }
	public void distrustCABasedOnDN(Session session,String dn) throws DistrustCAException{
                try{
                        Query query=(session.createQuery("update CA set trusted = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new DistrustCAException("Exception: DistrustCABasedOnDN: "+e.getMessage());
                }
        }
}
