/*****************************************************************************
*       File            RoleDaoImpl.java
*       Author          Neha Sharma
*       Date            August 26' 2009
*       Organization    Fermi National Accelerator Laboratory
*       Description     The implementation of Role DAO interface.
*       Email           neha@fnal.gov
******************************************************************************/
package gov.fnal.vox.saz.dao.impl.hibernate;

import java.util.List;
import gov.fnal.vox.saz.entity.Role;
import org.hibernate.Session;
import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.HibernateException;
import gov.fnal.vox.saz.dao.RoleDao;
import gov.fnal.vox.saz.exception.AddRoleException;
import gov.fnal.vox.saz.exception.BanRoleException;
import gov.fnal.vox.saz.exception.UnbanRoleException;
import gov.fnal.vox.saz.exception.TrustRoleException;
import gov.fnal.vox.saz.exception.DistrustRoleException;
import gov.fnal.vox.saz.exception.FindRoleException;
import gov.fnal.vox.saz.exception.ListRolesException;
import gov.fnal.vox.saz.exception.FindRoleUnexpectedException;
import org.hibernate.exception.ConstraintViolationException;

public class RoleDaoImpl extends BaseDaoImpl implements RoleDao{
	private static final transient Logger logger=Logger.getLogger(RoleDaoImpl.class.getName());
	public void add(Session session,String dn,String enabled,String trusted) throws AddRoleException{
		logger.debug("AddRole: Is Session Open: "+session.isOpen());
		try{
			Role newRole=new Role();
			newRole.setDn(dn);
			newRole.setEnabled(enabled);
			newRole.setTrusted(trusted);
			session.save(newRole);
		}catch(HibernateException e){
			throw new AddRoleException("Exception: Add New Role: "+e.getMessage());
 		}
	}
	public Role find(Session session,String dn) throws FindRoleException, FindRoleUnexpectedException{
		Role toReturn=null;
		logger.debug("FindRole: Is Session Open: "+session.isOpen());	
		try{
 			Query query=(session.createQuery("from Role where dn = :var"));	
			query.setString("var",dn);
			query.setCacheable(true);
			Object o=query.uniqueResult();
			if(o != null){
				if(o instanceof Role){
					toReturn=(Role)o;
					return toReturn;
				}else{
					throw new FindRoleUnexpectedException("Unexpected Exception: Find Role: ");
				}
			}else{
				return null;
			}
 		}catch(HibernateException e){
 			throw new FindRoleException("Exception: Find Role: "+e.getMessage());
		}
	}
	public List<Role> list(Session session) throws ListRolesException{
		List<Role> result=null;
		try{
			Query query=(session.createQuery("from Role"));
			result=query.list();
			return result;	
		}catch(HibernateException e){
			throw new ListRolesException("Exception: ListRoles: "+e.getMessage());
		}
	}
	public List<Role> listAllBannedRoles(Session session) throws ListRolesException{
		List<Role> result=null;
		try{
                        Query query=(session.createQuery("from Role where enabled = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListRolesException("Exception: ListAllBannedRoles: "+e.getMessage());
                }
        }
	public List<Role> listAllUnbannedRoles(Session session) throws ListRolesException{
                List<Role> result=null;
                try{
                        Query query=(session.createQuery("from Role where enabled = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result); 
                }catch(HibernateException e){
                        throw new ListRolesException("Exception: ListAllUnbannedRoles: "+e.getMessage());
                }
        }
	public List<Role> listAllTrustedRoles(Session session) throws ListRolesException{
                List<Role> result=null;
                try{
                        Query query=(session.createQuery("from Role where trusted = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListRolesException("Exception: ListAllTrustedRoles: "+e.getMessage());
                }
        }
	public List<Role> listAllDistrustedRoles(Session session) throws ListRolesException{
                List<Role> result=null;
                try{
                        Query query=(session.createQuery("from Role where trusted = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListRolesException("Exception: ListAllDistrustedRoles: "+e.getMessage());
                }
        }
	public void banRoleBasedOnDN(Session session,String dn) throws BanRoleException{
                try{
                        Query query=(session.createQuery("update Role set enabled = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new BanRoleException("Exception: BanRoleBasedOnDN: "+e.getMessage());
                }
        }
	public void unbanRoleBasedOnDN(Session session,String dn) throws UnbanRoleException{
                try{
                        Query query=(session.createQuery("update Role set enabled = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new UnbanRoleException("Exception: UnbanRoleBasedOnDN: "+e.getMessage());
                }
        }
	public void trustRoleBasedOnDN(Session session,String dn) throws TrustRoleException{
                try{
                        Query query=(session.createQuery("update Role set trusted = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new TrustRoleException("Exception: TrustRoleBasedOnDN: "+e.getMessage());
                }
        }
	public void distrustRoleBasedOnDN(Session session,String dn) throws DistrustRoleException{
                try{
                        Query query=(session.createQuery("update Role set trusted = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new DistrustRoleException("Exception: DistrustRoleBasedOnDN: "+e.getMessage());
                }
        }
}
