/*****************************************************************************
*       File            UserDaoImpl.java
*       Author          Neha Sharma
*       Date            August 26' 2009
*       Organization    Fermi National Accelerator Laboratory
*       Description     The implementation of User DAO interface.
*       Email           neha@fnal.gov
******************************************************************************/
package gov.fnal.vox.saz.dao.impl.hibernate;

import java.util.List;
import gov.fnal.vox.saz.entity.User;
import org.hibernate.Session;
import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.HibernateException;
import gov.fnal.vox.saz.dao.UserDao;
import gov.fnal.vox.saz.exception.AddUserException;
import gov.fnal.vox.saz.exception.BanUserException;
import gov.fnal.vox.saz.exception.UnbanUserException;
import gov.fnal.vox.saz.exception.TrustUserException;
import gov.fnal.vox.saz.exception.DistrustUserException;
import gov.fnal.vox.saz.exception.FindUserException;
import gov.fnal.vox.saz.exception.ListUsersException;
import gov.fnal.vox.saz.exception.FindUserUnexpectedException;
import org.hibernate.exception.ConstraintViolationException;

public class UserDaoImpl extends BaseDaoImpl implements UserDao{
	private static final transient Logger logger=Logger.getLogger(UserDaoImpl.class.getName());
	public void add(Session session,String dn,String enabled,String trusted) throws AddUserException{
		logger.debug("AddUser: Is Session Open: "+session.isOpen());
		try{
			User newUser=new User();
			newUser.setDn(dn);
			newUser.setEnabled(enabled);
			newUser.setTrusted(trusted);
			session.save(newUser);
		}catch(HibernateException e){
			throw new AddUserException("Exception: Add New User: "+e.getMessage());
 		}
	}
	public User find(Session session,String dn) throws FindUserException, FindUserUnexpectedException{
		User toReturn=null;
		logger.debug("FindUser: Is Session Open: "+session.isOpen());	
		try{
 			Query query=(session.createQuery("from User where dn = :var"));	
			query.setString("var",dn);
			query.setCacheable(true);
			Object o=query.uniqueResult();
			if(o != null){
				if(o instanceof User){
					toReturn=(User)o;
					return toReturn;
				}else{
					throw new FindUserUnexpectedException("Unexpected Exception: Find User: ");
				}
			}else{
				return null;
			}
 		}catch(HibernateException e){
 			throw new FindUserException("Exception: Find User: "+e.getMessage());
		}
	}
	public List<User> list(Session session) throws ListUsersException{
		List<User> result=null;
		try{
			Query query=(session.createQuery("from User"));
			result=query.list();
			return result;	
		}catch(HibernateException e){
			throw new ListUsersException("Exception: ListUsers: "+e.getMessage());
		}
	}
	public List<User> listAllBannedUsers(Session session) throws ListUsersException{
		List<User> result=null;
		try{
                        Query query=(session.createQuery("from User where enabled = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListUsersException("Exception: ListAllBannedUsers: "+e.getMessage());
                }
        }
	public List<User> listAllUnbannedUsers(Session session) throws ListUsersException{
                List<User> result=null;
                try{
                        Query query=(session.createQuery("from User where enabled = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result); 
                }catch(HibernateException e){
                        throw new ListUsersException("Exception: ListAllUnbannedUsers: "+e.getMessage());
                }
        }
	public List<User> listAllTrustedUsers(Session session) throws ListUsersException{
                List<User> result=null;
                try{
                        Query query=(session.createQuery("from User where trusted = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListUsersException("Exception: ListAllTrustedUsers: "+e.getMessage());
                }
        }
	public List<User> listAllDistrustedUsers(Session session) throws ListUsersException{
                List<User> result=null;
                try{
                        Query query=(session.createQuery("from User where trusted = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListUsersException("Exception: ListAllDistrustedUsers: "+e.getMessage());
                }
        }
	public void banUserBasedOnDN(Session session,String dn) throws BanUserException{
                try{
                        Query query=(session.createQuery("update User set enabled = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new BanUserException("Exception: BanUserBasedOnDN: "+e.getMessage());
                }
        }
	public void unbanUserBasedOnDN(Session session,String dn) throws UnbanUserException{
                try{
                        Query query=(session.createQuery("update User set enabled = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new UnbanUserException("Exception: UnbanUserBasedOnDN: "+e.getMessage());
                }
        }
	public void trustUserBasedOnDN(Session session,String dn) throws TrustUserException{
                try{
                        Query query=(session.createQuery("update User set trusted = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new TrustUserException("Exception: TrustUserBasedOnDN: "+e.getMessage());
                }
        }
	public void distrustUserBasedOnDN(Session session,String dn) throws DistrustUserException{
                try{
                        Query query=(session.createQuery("update User set trusted = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new DistrustUserException("Exception: DistrustUserBasedOnDN: "+e.getMessage());
                }
        }
}
