/*****************************************************************************
*       File            VODaoImpl.java
*       Author          Neha Sharma
*       Date            August 26' 2009
*       Organization    Fermi National Accelerator Laboratory
*       Description     The implementation of VO DAO interface.
*       Email           neha@fnal.gov
******************************************************************************/
package gov.fnal.vox.saz.dao.impl.hibernate;

import java.util.List;
import gov.fnal.vox.saz.entity.VO;
import org.hibernate.Session;
import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.HibernateException;
import gov.fnal.vox.saz.dao.VODao;
import gov.fnal.vox.saz.exception.AddVOException;
import gov.fnal.vox.saz.exception.BanVOException;
import gov.fnal.vox.saz.exception.UnbanVOException;
import gov.fnal.vox.saz.exception.TrustVOException;
import gov.fnal.vox.saz.exception.DistrustVOException;
import gov.fnal.vox.saz.exception.FindVOException;
import gov.fnal.vox.saz.exception.ListVOsException;
import gov.fnal.vox.saz.exception.FindVOUnexpectedException;
import org.hibernate.exception.ConstraintViolationException;

public class VODaoImpl extends BaseDaoImpl implements VODao{
	private static final transient Logger logger=Logger.getLogger(VODaoImpl.class.getName());
	public void add(Session session,String dn,String enabled,String trusted) throws AddVOException{
		logger.debug("AddVO: Is Session Open: "+session.isOpen());
		try{
			VO newVO=new VO();
			newVO.setDn(dn);
			newVO.setEnabled(enabled);
			newVO.setTrusted(trusted);
			session.save(newVO);
		}catch(HibernateException e){
			throw new AddVOException("Exception: Add New VO: "+e.getMessage());
 		}
	}
	public VO find(Session session,String dn) throws FindVOException, FindVOUnexpectedException{
		VO toReturn=null;
		logger.debug("FindVO: Is Session Open: "+session.isOpen());	
		try{
 			Query query=(session.createQuery("from VO where dn = :var"));	
			query.setString("var",dn);
			query.setCacheable(true);
			Object o=query.uniqueResult();
			if(o != null){
				if(o instanceof VO){
					toReturn=(VO)o;
					return toReturn;
				}else{
					throw new FindVOUnexpectedException("Unexpected Exception: Find VO: ");
				}
			}else{
				return null;
			}
 		}catch(HibernateException e){
 			throw new FindVOException("Exception: Find VO: "+e.getMessage());
		}
	}
	public List<VO> list(Session session) throws ListVOsException{
		List<VO> result=null;
		try{
			Query query=(session.createQuery("from VO"));
			result=query.list();
			return result;	
		}catch(HibernateException e){
			throw new ListVOsException("Exception: ListVOs: "+e.getMessage());
		}
	}
	public List<VO> listAllBannedVOs(Session session) throws ListVOsException{
		List<VO> result=null;
		try{
                        Query query=(session.createQuery("from VO where enabled = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListVOsException("Exception: ListAllBannedVOs: "+e.getMessage());
                }
        }
	public List<VO> listAllUnbannedVOs(Session session) throws ListVOsException{
                List<VO> result=null;
                try{
                        Query query=(session.createQuery("from VO where enabled = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result); 
                }catch(HibernateException e){
                        throw new ListVOsException("Exception: ListAllUnbannedVOs: "+e.getMessage());
                }
        }
	public List<VO> listAllTrustedVOs(Session session) throws ListVOsException{
                List<VO> result=null;
                try{
                        Query query=(session.createQuery("from VO where trusted = :var"));
                        query.setString("var","Y");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListVOsException("Exception: ListAllTrustedVOs: "+e.getMessage());
                }
        }
	public List<VO> listAllDistrustedVOs(Session session) throws ListVOsException{
                List<VO> result=null;
                try{
                        Query query=(session.createQuery("from VO where trusted = :var"));
                        query.setString("var","N");
                        result=query.list();
                        return(result);
                }catch(HibernateException e){
                        throw new ListVOsException("Exception: ListAllDistrustedVOs: "+e.getMessage());
                }
        }
	public void banVOBasedOnDN(Session session,String dn) throws BanVOException{
                try{
                        Query query=(session.createQuery("update VO set enabled = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new BanVOException("Exception: BanVOBasedOnDN: "+e.getMessage());
                }
        }
	public void unbanVOBasedOnDN(Session session,String dn) throws UnbanVOException{
                try{
                        Query query=(session.createQuery("update VO set enabled = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new UnbanVOException("Exception: UnbanVOBasedOnDN: "+e.getMessage());
                }
        }
	public void trustVOBasedOnDN(Session session,String dn) throws TrustVOException{
                try{
                        Query query=(session.createQuery("update VO set trusted = 'Y' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new TrustVOException("Exception: TrustVOBasedOnDN: "+e.getMessage());
                }
        }
	public void distrustVOBasedOnDN(Session session,String dn) throws DistrustVOException{
                try{
                        Query query=(session.createQuery("update VO set trusted = 'N' where dn = :var"));
                        query.setString("var",dn);
                        query.executeUpdate();
                }catch(HibernateException e){
                        throw new DistrustVOException("Exception: DistrustVOBasedOnDN: "+e.getMessage());
                }
        }
}
