package gov.fnal.vox.saz.service;
  
import java.util.List;
import java.util.Date;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import org.apache.log4j.Logger;
import gov.fnal.vox.saz.bl.SAZAPIImpl;
 
import org.opensaml.saml2.core.Statement;
import org.opensaml.saml2.core.impl.SubjectBuilder;
import org.opensaml.xml.XMLObjectBuilderFactory;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.ctx.DecisionType;
import org.opensaml.xacml.ctx.ResponseType;
import org.opensaml.xacml.ctx.StatusType;
import org.opensaml.xacml.ctx.StatusCodeType;
import org.opensaml.xacml.ctx.SubjectType;
import org.opensaml.xacml.ctx.ResultType;
import org.opensaml.xacml.ctx.ResourceType;
import org.opensaml.xacml.ctx.AttributeType;
import org.opensaml.xacml.ctx.AttributeValueType;
import org.opensciencegrid.authz.xacml.common.FQAN;
import org.opensaml.xacml.ctx.impl.DecisionTypeImplBuilder;
import org.opensaml.xacml.ctx.impl.StatusCodeTypeImplBuilder;
import org.opensaml.xacml.ctx.impl.StatusTypeImplBuilder;
import org.opensaml.xacml.ctx.impl.ResultTypeImplBuilder;
import org.opensaml.xacml.ctx.impl.ResponseTypeImplBuilder;
import org.opensaml.xacml.ctx.impl.AttributeValueTypeImpl;
import org.opensaml.xacml.policy.AttributeAssignmentType;
import org.opensaml.xacml.policy.EffectType;
import org.opensaml.xacml.policy.ObligationsType;
import org.opensaml.xacml.policy.ObligationType;
import org.opensaml.xacml.policy.impl.AttributeAssignmentTypeImplBuilder;
import org.opensaml.xacml.policy.impl.ObligationTypeImplBuilder;
import org.opensaml.xacml.policy.impl.ObligationsTypeImplBuilder;
import org.opensaml.xacml.profile.saml.XACMLAuthzDecisionQueryType;
import org.opensaml.xacml.profile.saml.XACMLAuthzDecisionStatementType;
import org.opensaml.xacml.profile.saml.impl.XACMLAuthzDecisionStatementTypeImplBuilder;
import org.opensaml.xml.XMLObject;
  
import org.opensciencegrid.authz.xacml.service.XACMLMappingService;
import org.opensciencegrid.authz.xacml.common.XACMLConstants;
import org.opensaml.xacml.ctx.AttributeValueType;
  
public class InvokeSAZAPIAuthZService implements XACMLMappingService{
	private static String ERROR="http://oasis/names/tc/xacml/1.0/status/error";
	private static String OK="http://oasis/names/tc/xacml/1.0/status/ok";
	private static final transient Logger logger=Logger.getLogger(InvokeSAZAPIAuthZService.class.getName());
	//Neha - 03-31-2010 
	//Should not be static because need a separate instance for each invokation
	//private static SAZAPIImpl saz=new SAZAPIImpl();
	private SAZAPIImpl saz=new SAZAPIImpl();
	// Since SAZ does not do mapping, rather checks if user is authorized, I override the function below
	public XACMLAuthzDecisionStatementType mapCredentials(XACMLAuthzDecisionQueryType xacmlQuery) throws Exception{
		String userVO=null;
                String userRole=null;
  		XMLObjectBuilderFactory builderFactory = org.opensaml.xml.Configuration.getBuilderFactory();
  		// Get information from request
  		RequestType request = xacmlQuery.getRequest();
  		String hostDn=getResourceAttributeValue(request, XACMLConstants.RESOURCE_X509_ID);
  		String userDn=getSubjectAttributeValue(request, XACMLConstants.SUBJECT_X509_ID);
		String caDn=getSubjectAttributeValue(request, XACMLConstants.SUBJECT_X509_ISSUER);
		String userCertSerialNumber;	
		String userProxyLifetime;
		String userCertNotBefore;
		String userCertNotAfter;
		long timediff=0;
		double infinity=1.0/0.0;
		//CA_CERT_SERIAL_NUM is present in constant SUBJECT_CA_SERIAL_NUMBER_ID	
		//There is another XACML constant that represents an FQAN. That allows one to send
		//all attributes present in user proxy one by one
		//Primary FQAN is what SAZ wants.
		String userFqan=getSubjectAttributeValue(request, XACMLConstants.SUBJECT_VOMS_PRIMARY_FQAN_ID);
		userVO=getSubjectAttributeValue(request, XACMLConstants.SUBJECT_VO_ID);
  		if(hostDn==null || hostDn.length()==0){
 			logger.debug("missing attribute: "+XACMLConstants.RESOURCE_X509_ID);
 			throw new Exception("missing attribute: "+XACMLConstants.RESOURCE_X509_ID);
 		}
  		if(userDn==null || userDn.length()==0){
  			logger.debug("missing attribute: "+XACMLConstants.SUBJECT_X509_ID);
  			throw new Exception("missing attribute: "+XACMLConstants.SUBJECT_X509_ID);
 		}
		if(caDn==null || caDn.length()==0){
                        logger.debug("missing attribute: "+XACMLConstants.SUBJECT_X509_ISSUER);
                        throw new Exception("missing attribute: "+XACMLConstants.SUBJECT_X509_ISSUER);
                }
		//Role is FQAN itself
		if(userFqan!=null){
			FQAN fqan=new FQAN(userFqan);
			userRole=fqan.toString();
		}
		if(userVO==null){
			userVO="NULL";
		}
		if(userRole==null){
                        userRole="NULL";
                }else{
			if(userRole.indexOf('/') > 0){
				userRole="/"+userRole;
			}
		}
		//If SAZ Admin has chosen to ban users based on their certificate serial number, then modify the
		//user DN accordingly
		if(saz.getIsUserCertSerialNumBanEnabled().equals("Y")){
                        userCertSerialNumber=getSubjectAttributeValue(request, XACMLConstants.SUBJECT_CERTIFICATE_SERIAL_NUMBER_ID);
			if(userCertSerialNumber==null || userCertSerialNumber.length()==0){
                        	logger.debug("missing attribute: "+XACMLConstants.SUBJECT_CERTIFICATE_SERIAL_NUMBER_ID);
                        	throw new Exception("missing attribute: "+XACMLConstants.SUBJECT_CERTIFICATE_SERIAL_NUMBER_ID);
                	}else{
				userDn=userDn.concat("/SERIAL=");
				userDn=userDn.concat(userCertSerialNumber);
			}
		}
		//For each request, try to get the proxy lifetime attribute values. If not sent my client, set proxy lifetime to infinity
		Date notBefore=new Date();
		Date notAfter=new Date();
		boolean attrsMissing=false;
		//if(saz.getIsUserProxyLifetimeBanEnabled().equals("Y")){
		DatatypeFactory dataFactory = DatatypeFactory.newInstance();
		userCertNotBefore=getSubjectAttributeValue(request, XACMLConstants.SUBJECT_CERTIFICATE_NOT_BEFORE_ID);
		userCertNotAfter=getSubjectAttributeValue(request, XACMLConstants.SUBJECT_CERTIFICATE_NOT_AFTER_ID);
		if(userCertNotBefore==null || userCertNotBefore.length()==0){
			attrsMissing=true;
			logger.info("missing attribute: "+XACMLConstants.SUBJECT_CERTIFICATE_NOT_BEFORE_ID);
			//throw new Exception("missing attribute: "+XACMLConstants.SUBJECT_CERTIFICATE_NOT_BEFORE_ID);
		}else{
			notBefore=dataFactory.newXMLGregorianCalendar(userCertNotBefore).toGregorianCalendar().getTime();
		}
		if(userCertNotAfter==null || userCertNotAfter.length()==0){
			attrsMissing=true;
			logger.info("missing attribute: "+XACMLConstants.SUBJECT_CERTIFICATE_NOT_AFTER_ID);
                        //throw new Exception("missing attribute: "+XACMLConstants.SUBJECT_CERTIFICATE_NOT_AFTER_ID);
                }else{
			notAfter=dataFactory.newXMLGregorianCalendar(userCertNotAfter).toGregorianCalendar().getTime();
		}
		//logger.info("User "+userDn);
                //logger.info("CA "+caDn);
                //logger.info("VO "+userVO);
                //logger.info("Role "+userRole);
		if(attrsMissing){
			logger.info("Setting user proxy lifetime to infinity");
			timediff=Math.round(infinity);
			logger.info("NotBefore: Missing"+" - NotAfter: Missing"+" - Incoming Proxy Lifetime: infinity seconds");
		}else{
			if(notAfter.compareTo(notBefore) < 0){
				timediff=Math.round(infinity);
				logger.info("User Proxy NotBefore date cannot be after the NotAfter date.");
				logger.info("Setting user proxy lifetime to infinity");
				logger.info("NotBefore: "+notBefore.toString()+" - NotAfter: "+notAfter.toString()+" - Incoming Proxy Lifetime: infinity seconds");
			}else{
				timediff=(notAfter.getTime() - notBefore.getTime())/1000;
				logger.info("NotBefore: "+notBefore.toString()+" - NotAfter: "+notAfter.toString()+" - Incoming Proxy Lifetime: " +timediff+" seconds");
			}
		}
  		// Attribute Assignment, decision, and status code
  		AttributeAssignmentType attributeAssignment=null;
  		DecisionTypeImplBuilder decisionBuilder=(DecisionTypeImplBuilder)builderFactory.getBuilder(DecisionType.DEFAULT_ELEMENT_NAME);
  		DecisionType decision=decisionBuilder.buildObject();
  		StatusCodeTypeImplBuilder statusCodeBuilder=(StatusCodeTypeImplBuilder)builderFactory.getBuilder(StatusCodeType.DEFAULT_ELEMENT_NAME);
  		StatusCodeType statusCode=statusCodeBuilder.buildObject();
  		statusCode.setValue(OK);
  		try{
			//Need to insert/update the user information in DB
			String sazDecision;
			sazDecision=saz.isEnabled(userDn,caDn,userVO,userRole);	
			//If banning based on user proxy lifetime is enabled, and user proxy lifetime does exceed
			//maximum configured, set SAZ decision to "N"
			if(saz.getIsUserProxyLifetimeBanEnabled().equals("Y")){
				if(saz.compareProxyLifetime(timediff)){
					logger.info("Incoming Proxy Lifetime exceeds the maximum allowed by SAZ Administrator");
					sazDecision = "N";
				}
			}
			logger.info("Sending to client "+sazDecision);
			if (sazDecision == "N"){
  				decision.setDecision(DecisionType.DECISION.Deny);
  				logger.debug("Denied access to user " + userDn + "' with fqan '" + userFqan + "'");
  			}
  			else{
 				decision.setDecision(DecisionType.DECISION.Permit);
 				logger.debug("Allowed access to user " + userDn + "' with fqan '" + userFqan + "'");
			}
 		}catch(Exception e1){
 			statusCode.setValue(ERROR);
 			logger.debug(e1.getMessage());
 			throw e1;
 		}
 		try{
 			StatusTypeImplBuilder statusBuilder = (StatusTypeImplBuilder)builderFactory.getBuilder(StatusType.DEFAULT_ELEMENT_NAME);
 			StatusType status = statusBuilder.buildObject();
 			status.setStatusCode(statusCode);
 			// Result
 			ResultTypeImplBuilder resultBuilder = (ResultTypeImplBuilder)builderFactory.getBuilder(ResultType.DEFAULT_ELEMENT_NAME);
 			ResultType result = resultBuilder.buildObject();
			result.setStatus(status);
 			result.setDecision(decision);
 			// Response      
 			ResponseTypeImplBuilder responseBuilder = (ResponseTypeImplBuilder)builderFactory.getBuilder(ResponseType.DEFAULT_ELEMENT_NAME);
 			ResponseType response = responseBuilder.buildObject();
 			response.setResult(result);
 			// Statement
 			XACMLAuthzDecisionStatementTypeImplBuilder xacmlauthzBuilder = (XACMLAuthzDecisionStatementTypeImplBuilder)builderFactory.getBuilder(XACMLAuthzDecisionStatementType.TYPE_NAME_XACML20);
 			XACMLAuthzDecisionStatementType xacmlAuthzStatement = xacmlauthzBuilder.buildObject( Statement.DEFAULT_ELEMENT_NAME, XACMLAuthzDecisionStatementType.TYPE_NAME_XACML20);	
 			if (xacmlQuery.getReturnContextXSBooleanValue() != null && xacmlQuery.getReturnContextXSBooleanValue().getValue()) 
 			{
 				request.detach();
 				xacmlAuthzStatement.setRequest(request);
 			}
 			xacmlAuthzStatement.setResponse(response);
 			return xacmlAuthzStatement;
 		} catch (Exception e1) {
 			statusCode.setValue(ERROR);
 			logger.debug(e1.getMessage());
 			throw e1;
 		}
 	}
	private String getSubjectAttributeValue(RequestType request, String attributeId) {
		List<SubjectType> subjectList = request.getSubjects();
		for(SubjectType subject : subjectList) {
 			List<AttributeType> attributeList = subject.getAttributes();
 			for(AttributeType attribute : attributeList) {
 				String curAttributeId = attribute.getAttributeID();
 				if (attributeId.equals(curAttributeId)) {
 					List<AttributeValueType> attributeValueList = attribute.getAttributeValues();
 					for(AttributeValueType attributeValue : attributeValueList) {
 						return ((AttributeValueTypeImpl)attributeValue).getValue();
 					}
 				}
 			}
 		}  
 		return null;
 	}
 	private String getResourceAttributeValue(RequestType request, String attributeId) {
 		List<ResourceType> resourceList = request.getResources();
 		for(ResourceType resource : resourceList) {
 			List<AttributeType> attributeList = resource.getAttributes();
 			for(AttributeType attribute : attributeList) {
 				String curAttributeId = attribute.getAttributeID();
 				if (attributeId.equals(curAttributeId)) {
 					List<AttributeValueType> attributeValueList = attribute.getAttributeValues();
 					for(AttributeValueType attributeValue : attributeValueList) {
 						return ((AttributeValueTypeImpl)attributeValue).getValue();
 					}
 				}
 			}
 		}  
 		return null;
 	}

 }

